@@ -96,7 +96,7 @@ def make_db_from_schema():
         ('classic',
          'http://37.251.146.169:8000/streamHD',
          save_parent_folder,
-         'ro - hq stream',
+         '',
          "image/png",
          convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
         )
@@ -106,7 +106,7 @@ def make_db_from_schema():
         ('goa_psi',
          'http://amoris.sknt.ru:8004/stream',
          save_parent_folder,
-         'ru - Trance Psychedelic Experimental',
+         '',
          "image/png",
          convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
         )
@@ -120,16 +120,6 @@ def make_db_from_schema():
          "image/png",
          convert_ascii(static_images + "radio3d-style-white-blue_120x200.png"))
         )
-    cur.execute(
-        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
-        "?, ?, ?)",
-        ('korean_pop',
-         'http://167.114.64.181:8818/stream',
-         save_parent_folder,
-         '',
-         "image/png",
-         convert_ascii(static_images + "radio3d-style-white-violet_120x200.png"))
-        )
     cur.execute(
         "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
         "?, ?, ?)",
@@ -166,7 +156,7 @@ def make_db_from_schema():
         ('Nachtflug',
          'http://85.195.88.149:11810/sid=1',
          save_parent_folder,
-         'added sid=1, http://85.195.88.149:11810/sid=1',
+         '',
          "image/png",
          convert_ascii(static_images + "radio3d-style-white-red_120x200.png"))
         )
