@@ -1,10 +1,10 @@
-var audio_visual, audioContext, audioSource, analyser;
+var audioContext, audioSource, analyser;
 var spectrum_analyser = false;
 
 function setAudioContextVisual() {
-    audio_visual.volume = 0.25;
+    audio = document.getElementById("audio_with_controls");
     audioContext = new AudioContext();
-    audioSource = audioContext.createMediaElementSource(audio_visual);
+    audioSource = audioContext.createMediaElementSource(audio);
     analyser = audioContext.createAnalyser();
     audioSource.connect(analyser).connect(audioContext.destination);
     // const prime_button = document.querySelectorAll('.btn-primary');
@@ -30,21 +30,14 @@ $(document).ready(function () {
     setInterval(updateDisplay, 5001);
 
     const file_upload = document.getElementById("fileupload");
-    const form_play_local = document.getElementById("form_play_local");
     const local_host_sound_route = "http://localhost:5050/sound/"
-    const local_host_sound_route_upload = "http://localhost:5050/sound_local"
     const play_btn = document.getElementById("play_btn");
     const pause_btn = document.getElementById("pause_btn");
-    var audio = document.getElementById("audio_with_controls");
-    var audio_volume_control = document.getElementById("audio_volume_controller");
+    const audio = document.getElementById("audio_with_controls");
     audio.volume = 0.25;
+    const audio_volume_control = document.getElementById("audio_volume_controller");
 
-    var analyser_random = Math.random();
-    if (analyser_random < 0.5) {
-        analyser_random = 0
-    } else {
-        analyser_random = 1
-    }
+    var analyser_random = getRandomIntInclusive(1,2);
 
     audio.addEventListener("play", function () {
         play_btn.style.display = "none";
@@ -65,6 +58,8 @@ $(document).ready(function () {
         });
     });
 
+    audio_volume_control.addEventListener("input", setAudioVolume);
+
     file_upload.addEventListener("change", function () {
         var audio_duration;
         const playlist_title = document.getElementById("playlist_title");
@@ -79,10 +74,8 @@ $(document).ready(function () {
             document.getElementById('file_list').scrollIntoView({ behavior: "smooth" });
         });
 
-        // playlist_title.style.border = ".1em solid var(--border-color)";
-
-        console.log(this.files);
-        const files = this.files;    // inbound who knows
+        // console.log(this.files);
+        const files = this.files;
 
         const clone_files = [...files];
         const audio = document.getElementById("audio_with_controls");
@@ -100,18 +93,18 @@ $(document).ready(function () {
         audio.src = URL.createObjectURL(play_list[0]);
         playlist_title.value = play_list[0].name;
 
-            /*
-        calculateMediaDuration(audio).then(() => {
-            console.log(audio.duration);
-        });
-        */
+        /*
+    calculateMediaDuration(audio).then(() => {
+        console.log(audio.duration);
+    });
+    */
         audio.load();
         audio.play;
         // setTimeout(() => {
         //     audio.pause();
         //     audio.currentTime = Number.MAX_SAFE_INTEGER; // chrome bug; NaN, infinity
         // }, audio_duration);
-        if (!spectrum_analyser){
+        if (!spectrum_analyser) {
             visualise_audio(null);    // next version gets a second colored spec analyser in each comments table cell
             spectrum_analyser = true;
         }
@@ -123,7 +116,6 @@ $(document).ready(function () {
                 audio.srcObject = null;
                 audio.src = URL.createObjectURL(play_list[track]);
                 playlist_title.value = play_list[track].name;
-
                 audio.load();
                 audio.play();
 
@@ -133,9 +125,6 @@ $(document).ready(function () {
 
     });  // file_up
 
-    audio_volume_control.addEventListener("input", setAudioVolume);
-    audio_visual = document.getElementById("audio_with_controls");
-
     function is_play_station() {
         let req = $.ajax({
             type: 'GET',
@@ -147,12 +136,12 @@ $(document).ready(function () {
             let player_id = data.is_play_station_id;
             let player_name = data.is_play_station_name;
             if (player_id !== '-empty-') {
-
-                audio_visual.src = local_host_sound_route + player_name;
-                audio_visual.play();
-                if (!spectrum_analyser){
+                audio.src = local_host_sound_route + player_name;
+                audio.load();
+                audio.play();
+                if (!spectrum_analyser) {
                     visualise_audio(null);
-                spectrum_analyser = true;
+                    spectrum_analyser = true;
                 }
             }
         });
@@ -270,18 +259,11 @@ $(document).ready(function () {
 
             if (data.result === 'deactivate_audio') {
                 console.log('deactivate_audio');
-
-                audio_control = 'audio_with_controls';
-                myAudio = document.getElementById(audio_control);
-                myAudio.src = "";
-                myAudio.pause();
-                myAudio.currentTime = 0;
-                //There is no stop method — to implement a stop function,
-                //you'd have to pause the media then set the currentTime property value to 0.
-                myAudio.srcObject = null;
+                audio.src = "";
+                audio.currentTime = 0;
+                audio.srcObject = null;
                 document.getElementById('lbl_div_audio').innerText = '';
                 cookie_del_station();
-
             }
             ;
 
@@ -293,16 +275,17 @@ $(document).ready(function () {
                     analyser.connect(audioContext.destination);
                 } catch (error) {
                     console.error(error);
-                }
-                audio_control = 'audio_with_controls';
-                audio_visual = document.getElementById(audio_control);
-                audio_visual.src = "";   // remove from mem
-                audio_visual.pause();
-                audio_visual.currentTime = 0;
-                audio_visual.load();
-                audio_visual.src = local_host_sound_route + current_station;
-                audio_visual.play();
-                if (!spectrum_analyser){
+                };
+                audio.volume = 0.25;
+                audio_volume_control.value = 25;
+                audio.src = "";
+                audio.currentTime = 0;
+                audio.srcObject = null;
+                audio.src = local_host_sound_route + current_station;
+                audio.load();
+                audio.play();
+
+                if (!spectrum_analyser) {
                     visualise_audio(null);
                     spectrum_analyser = true;
                 }
@@ -393,31 +376,54 @@ $(document).ready(function () {
         const canvas = document.getElementById("canvas_master");
         // console.log('show canvas: ' + var_canvas)
         var canvas_ctx = canvas.getContext('2d');
+        analyser.fftSize = 128;
+        const bufferLength = analyser.frequencyBinCount;
+        var barWidth = (canvas.width / bufferLength) * 2; // const barWidth = (canvas.width/2) / bufferLength;
+        const dataArray = new Uint8Array(bufferLength);
+        let barHeight;
+        let x = 0;
 
+        rotateVisualiser();
+        function rotateVisualiser() {
+            barWidth = (canvas.width/2) / bufferLength;
+            requestAnimationFrame(rotateVisualiser);
+            x = 0;
+            analyser.getByteFrequencyData(dataArray);
+            canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
+            for (let i = 0; i < bufferLength; i++){
+                barHeight = dataArray[i] * 0.36;
+
+                canvas_ctx.save();
+                canvas_ctx.translate(canvas.width/2, canvas.height/2);
+                canvas_ctx.rotate(i * Math.PI * 4 / bufferLength);
+                const hue = i * 5;
+                canvas_ctx.fillStyle = 'hsl(' + hue + ',100%,50%)';
+                canvas_ctx.fillRect(0, 0, barWidth, barHeight);
+                x += barWidth;
+                canvas_ctx.restore();
+            }
+        }
         // animate();
         function animate() {
-            requestAnimationFrame(animate);
-
-            analyser.fftSize = 64;
-            const bufferLength = analyser.frequencyBinCount;
-            const dataArray = new Uint8Array(bufferLength);
+            x = 0;
+            analyser.fftSize = 128;
             analyser.getByteFrequencyData(dataArray);
-
-            const barWidth = (canvas.width / bufferLength) * 2.5;
-            let barHeight;
-            let x = 0;
-
             canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
-            // function drawVisualizer(bufferLength, x, barWidth, barHeight, dataArray);
-            for (let i = 0; i < bufferLength; i++) {
+            requestAnimationFrame(animate);
+            for (let i = 0; i < bufferLength; i++){
                 barHeight = (dataArray[i] / 2) + 2;
-                canvas_ctx.fillStyle = '#565454fc';  //  #d441fc
+                canvas_ctx.fillStyle = 'rgb(219, 111, 52)';  //  #d441fc
+                canvas_ctx.fillRect(x, canvas.height - barHeight - 10, barWidth, 2);
+                const red = i * barHeight / 4;
+                const green = i * 3;
+                const blue = barHeight / 2;
+                canvas_ctx.fillStyle = 'rgb('+ red + ',' + green + ',' + blue +')';
                 canvas_ctx.fillRect(x, canvas.height - barHeight, barWidth, barHeight);
                 x += barWidth;
             }
         }
 
-        // draw();
+        draw();
         function draw() {
             requestAnimationFrame(draw);
 
@@ -427,11 +433,15 @@ $(document).ready(function () {
             analyser.getByteTimeDomainData(dataArray);
 
             canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
-            canvas_ctx.lineWidth = 4;
-            canvas_ctx.strokeStyle = "#565454fc";
-
+            let darkBody = getBodyColor();
+            if (darkBody) {
+                canvas_ctx.lineWidth = 1.5;
+                canvas_ctx.strokeStyle = 'rgb(219, 111, 52)'; /*"#565454fc";*/
+            } else {
+                canvas_ctx.lineWidth = 3;
+                canvas_ctx.strokeStyle = '#565454fc';
+            }
             canvas_ctx.beginPath();
-
             var sliceWidth = canvas.width * 1.0 / bufferLength;
             var x = 0;
 
@@ -439,7 +449,6 @@ $(document).ready(function () {
 
                 var v = dataArray[i] / 128.0;
                 var y = v * canvas.height / 2;
-                // var y = v * canvas.height / 2;
 
                 if (i === 0) {
                     canvas_ctx.moveTo(x, y);
@@ -453,9 +462,10 @@ $(document).ready(function () {
             canvas_ctx.lineTo(canvas.width, canvas.height / 2);
             canvas_ctx.stroke();
         }
-        try {
-            if (analyser_random === 0) { animate(); }
-            if (analyser_random === 1) { draw(); }
+
+          try {
+            if (analyser_random === 1) { rotateVisualiser();}
+            if (analyser_random === 2) { draw();}
         } catch (error) {
             console.error(error);
         }
@@ -668,15 +678,17 @@ function cookie_toggle_show_visuals() {
     req.done(function (data) {
         let analyzer_badge = document.getElementById('analyzer_badge');
         let canvas = document.getElementById('canvas_master');
+        let div_canvas = document.getElementById('div_canvas_master');
         let show_visuals = data.str_visuals;
-        if (show_visuals !== 'show_visuals') {
-            analyzer_badge.textContent = "hide";
-            canvas_master.style.display = "block";
+        if (show_visuals !== 'show_visuals') {            analyzer_badge.textContent = "hide";
+            canvas.style.display = "inline-block";
+            div_canvas.style.display = "inline-block";
             cookie_set_show_visuals();
         }
         if (show_visuals === 'show_visuals') {
             analyzer_badge.textContent = "show";
-            canvas_master.style.display = "none";
+            canvas.style.display = "none";
+            div_canvas.style.display = "none";
             cookie_del_show_visuals();
 
         }
@@ -697,16 +709,19 @@ function cookie_start_set_text_show_visuals() {
 
     req.done(function (data) {
         let analyzer_badge = document.getElementById('analyzer_badge');
+        let div_canvas = document.getElementById('div_canvas_master');
         let canvas = document.getElementById('canvas_master');
         let show_visuals = data.str_visuals;
         if (show_visuals === 'show_visuals') {
             analyzer_badge.textContent = "hide";
-            canvas.style.display = "block";
+            canvas.style.display = "inline-block";
+            div_canvas.style.display = "inline-block";
 
         }
         if (show_visuals !== 'show_visuals') {
             analyzer_badge.textContent = "show";
             canvas.style.display = "none";
+            div_canvas.style.display = "none";
         }
 
 
@@ -771,7 +786,7 @@ function setColor(val) {
             bodyStyles.setProperty('--ghetto-measurements-bottom-color', '#FCA841');
             bodyStyles.setProperty('--radio-station-headline', '#4195fc');
             bodyStyles.setProperty('--controls-background', 'rgba(26,26,26,1)');
-            bodyStyles.setProperty('--canvas_master', 'rgba(26,26,26,0.7)');
+            bodyStyles.setProperty('--canvas_master', 'rgba(26,26,26,0.85)');
 
             setDarkmode();
         }
@@ -792,7 +807,7 @@ function setColor(val) {
             bodyStyles.setProperty('--ghetto-measurements-bottom-color', '#d441fc');
             bodyStyles.setProperty('--radio-station-headline', '#565454');
             bodyStyles.setProperty('--controls-background', '#565454');
-            bodyStyles.setProperty('--canvas_master', 'rgba(240, 240, 240, 0.7)');
+            bodyStyles.setProperty('--canvas_master', '#ccc');    // rgba(240, 240, 240, 0.85)
 
             delDarkmode();
         }
@@ -830,12 +845,15 @@ function meta_info() {
                 let station_name = val[3];
                 let station_id = val[4];
                 let bit_rate = val[5];
+                let icy_url = val[6];
 
                 document.getElementById('request_time_' + station_id).innerText = "" + response_time + " ms";
                 document.getElementById('request_suffix_' + station_id).innerText = "" + suffix;
                 document.getElementById('request_icy_br_' + station_id).innerText = "" + bit_rate + " kB/s";
                 document.getElementById('icy_name_' + station_id).innerText = "" + station_name;
                 document.getElementById('request_icy_genre_' + station_id).innerText = "" + genre;
+                document.getElementById('request_icy_url_' + station_id).innerText = "" + icy_url;
+                document.getElementById('request_icy_url_' + station_id).value = "" + icy_url;
             });
         }   /*data.cache_result !== ""*/
     });
@@ -861,6 +879,7 @@ function delete_info() {
                 document.getElementById('request_time_' + station_id).innerText = "";
                 document.getElementById('request_suffix_' + station_id).innerText = "";
                 document.getElementById('request_icy_br_' + station_id).innerText = "";
+                document.getElementById('request_icy_url_' + station_id).innerText = "";
                 document.getElementById('icy_name_' + station_id).innerText = "";
                 document.getElementById('request_icy_genre_' + station_id).innerText = "";
                 document.getElementById("canvas_" + station_id).style.display = "none";
@@ -957,4 +976,23 @@ function calculateMediaDuration(media) {
         }
     });
 }
-;
\ No newline at end of file
+;
+
+function getBodyColor() {
+    let bodyStyle = window.getComputedStyle(document.body, null);
+    let backgroundColor = bodyStyle.backgroundColor;
+    if (backgroundColor === 'rgb(26, 26, 26)') {
+        var darkBody = true;
+    } else { darkBody = false; }
+    return darkBody;
+}
+;
+
+function getRandomIntInclusive(min, max) {
+  min = Math.ceil(min);
+  max = Math.floor(max);
+  return Math.floor(Math.random() * (max - min +1)) + min;
+}
+;
+
+
