@@ -8,6 +8,8 @@ def stream_audio_feed(name):
     while True:
 
         try:
+            if not ghettoApi.listen_active_dict[name]:
+                return
             """grab from queue"""
             if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
                 chunk = ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
