@@ -1,5 +1,6 @@
 import os
-from flask import flash, redirect, url_for
+from flask import flash
+from eisenradio.api import ghettoApi
 
 
 def make_folder(this_folder):
@@ -41,3 +42,34 @@ def eisen_write_test(write_file):
     except OSError:  # master of desaster
         return False
     return True
+
+
+def is_in_db_view(a_name):
+    for radio_name in ghettoApi.radios_in_view_dict.values():
+        if radio_name == a_name:
+            return True
+    return False
+
+
+def remove_blank(a_name):
+    rv = a_name.replace(" ", "")
+    return rv
+
+
+def remove_special_chars(str_name):
+    str_name.replace(" ", "")
+    # my_str = "hey th~!ere. /\ coolleagues?! Straße"
+    rv = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\|`~=+"""'})
+    return rv
+
+
+# def get_stream_content_type(name):
+#     content_type = ''
+#     while not content_type:
+#         try:
+#             content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
+#             os.environ[name] = content_type
+#         except KeyError:
+#             pass
+#         sleep(.1)
+#     return content_type
