@@ -97,9 +97,12 @@ def meta(name):
         request_icy_genre_api(name),
         request_icy_name_api(name),
         request_icy_view_id_api(name),
-        request_icy_br_api(name)
+        request_icy_br_api(name),
+        request_icy_url_api(name)
     ]
-    # list order: request_time, request_suffix, request_icy_genre, request_icy_name, request_icy_view_id, request_icy_br
+    # list order:
+    # request_time, request_suffix, request_icy_genre, request_icy_name,
+    # request_icy_view_id, request_icy_br, request_icy_url
     return meta_info_list
 
 
@@ -167,3 +170,12 @@ def request_icy_br_api(name):
     except KeyError:
         pass
     return str(rv)
+
+
+def request_icy_url_api(name):
+    rv = '---'
+    try:
+        rv = ghettoApi.ghetto_measure_dict[name + ',icy_url']
+    except KeyError:
+        pass
+    return str(rv)
