@@ -40,8 +40,8 @@ from eisenradio.api import ghettoApi
 # logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 # logging.basicConfig(level=logging.INFO, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 
-version = '0.9.0'
-print(f'{version} ghettorecorder (eisen_radio module)')  # ########## version #############
+version = '1.4'
+print(f'ghettorecorder {version} (eisen_radio module)')  # ########## version #############
 
 
 class GBase:
@@ -74,16 +74,13 @@ class GBase:
 
     @staticmethod
     def remove_special_chars(str_name):
-        # cleanup for writing files and folders
-
-        # my_str = "hey th~!ere. /\ coolleagues?! Straße"
         ret_value = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\|`~=+"""'})
         return ret_value
 
     @staticmethod
     def this_time():
-        time = strftime("%Y-%m-%d %H:%M:%S")
-        return time
+        time_val = strftime("%Y-%m-%d %H:%M:%S")
+        return time_val
 
     def countdown(self, instance_attr_time):
         t = 0
@@ -106,7 +103,6 @@ class GIni(GBase):  # remnant of command line version
     # ini_key + '_single_title', ini_key + '_rec_from_here'
     cost_current_ini = ''  # ini key for cost_dict calc / should be a dict
     cost_dict = {}  # stores len of received headers to calc amount of data searching strings per day
-    fail_meta_dict = {}  # can not read metadata from stream, no data
     # list of search strings delimiter blank, first key is named 'STRINGS': Britney Phantom ไม่เคยจะจำ Elton Jim techno
     search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
     list_items = []  # radio key names for display in terminal
@@ -125,7 +121,7 @@ class GIni(GBase):  # remnant of command line version
 
 
 class GNet(GBase):
-    http_pool = urllib3.PoolManager(num_pools=200)
+    http_pool = urllib3.PoolManager(num_pools=20)
 
     @staticmethod
     def load_url(url):
@@ -141,13 +137,12 @@ class GNet(GBase):
         # we have server up, but content not presented - zombie
         try:
             GNet.load_url(url)
-        except HTTPError:  # lots of strange configured web server
-
+        except HTTPError:
             print('HTTPError')
             GBase.dict_error[str_radio] = 'HTTPError - check radio homepage'
             return True
         except URLError as error:  # <urlopen error timed out>
-            print(f' ---> {str_radio} server failed: {error} (no recording) {url}')
+            print(f' ---> {str_radio} server failed: {error} , {url}')
             GBase.dict_error[str_radio] = error
             print('URLError')
             return False
@@ -189,27 +184,33 @@ class GNet(GBase):
 
 class GRecorder:
     unknown_title_name = 'unknown_title'
-    path_to_song_dict = {}  # {station : file path}
+    path_record_dict = {}  # {station : file path}
     current_song_dict = {}  # each thread writes the new title to the station key name {station : title}
     start_write_command = {}  # recorder head thread set command to copy first file
     search_pattern_found = {}
 
     record_active_dict = {}
     listen_active_dict = {}
+    ghettoApi.init_ghetto_listen_active_dict(listen_active_dict)
     ghetto_measure = {}
     ghettoApi.init_ghetto_measurements(ghetto_measure)
     audio_stream_queue_dict = {}
     ghettoApi.init_ghetto_audio_stream(audio_stream_queue_dict)
 
-    @staticmethod  # copy keep-alive timeout=3000
-    def ghetto_recorder_display_title(url, str_radio):
+    @staticmethod
+    def ghetto_recorder_display_title(str_radio, str_action):
+        """display on terminal"""
         # print(f' ghetto_recorder_display_title: {str_radio}')
         update_terminal = ''
-        stream_song_name = GRecorder.current_song_dict[str_radio]
-        GIni.fail_meta_dict[url] = 'False'  # message display or not
+        stream_song_name = ''
+        stop_while = False
+        try:
+            stream_song_name = GRecorder.current_song_dict[str_radio]
+        except KeyError:
+            pass
         GRecorder.search_pattern_found[str_radio] = False
 
-        while not GBase.dict_exit[str_radio]:
+        while 1:
 
             if not len(stream_song_name) <= 2:
                 #  GRecorder.current_song_dict[str_radio] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
@@ -247,14 +248,21 @@ class GRecorder:
                 print(ex)
 
             for sec in range(GBase.sleeper):
+                if str_action == "listen":
+                    if GBase.dict_exit[str_radio] or not GRecorder.listen_active_dict[str_radio]:
+                        stop_while = True
+                        break
+                if str_action == "record":
+                    if GBase.dict_exit[str_radio] or not GRecorder.record_active_dict[str_radio]:
+                        stop_while = True
+                        break
                 sleep(1)
-                if GBase.dict_exit[str_radio]:
-                    # print(f' stop get title {str_radio}')
-                    break
+            if stop_while:
+                break
 
     @staticmethod
     def search_pattern_start_record(title, str_radio):
-        # search title for strings to see if we should start recording this
+        # search strings in title to see if we should start recording, test it in tk/tcl in ghettorecorder.win package
         search_list = []  # chop the string
 
         try:
@@ -275,88 +283,56 @@ class GRecorder:
         return False
 
     @staticmethod
-    def ghetto_recorder_head(directory_save, stream_suffix, radio_short_key):
-        # target: tail works without brain
-        # print(f' ghetto_recorder_head -- {radio_short_key}')
+    def ghetto_recorder_head(directory_save, stream_suffix, str_radio, str_action):
+        """create file name from current meta for recorder"""
+        if str_action == "listen":
+            return
 
-        sleep(GBase.sleeper)
-        # reader must work (conn. url) for writing the path from title name, can have a loop here
-        fresh_song = 'False'
+        GRecorder.start_write_command[str_radio] = False
+        sleep(3)  # chance to get stream meta info
         first_record = True
-        brake_loop = False
-        GRecorder.start_write_command[radio_short_key] = False
 
-        stream_song_name = GRecorder.current_song_dict[radio_short_key]
+        display_info = ""
+        while GRecorder.record_active_dict[str_radio] or not GBase.dict_exit[str_radio]:
+            stream_info = GRecorder.current_song_dict[str_radio]  # unknown_title, None, icy metadata if avail
 
-        try:
-            GBase.make_directory(GBase.radio_base_dir)
-        except FileNotFoundError:
-            GRecorder.current_song_dict[radio_short_key] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
-            GBase.dict_exit[radio_short_key] = True  # all radio_short_key treads-will-stop
-            print('\t---> Recorder Write Failure in : ' + GBase.radio_base_dir)
-            return
-
-        try:
-            GBase.make_directory(GBase.radio_base_dir + '//' + radio_short_key)
-        except OSError:
-            # "heyth~!ere./\coolleagues?!Straße"
-            GRecorder.current_song_dict[radio_short_key] = '[{-_-}] ZZZzz  Recorder Failure !!! DIRECTORY !!!'
-            print('\t---> Recorder Write Failure in : ' + GBase.radio_base_dir + '//' + radio_short_key)
-            sleep(5)
-            GBase.dict_exit[radio_short_key] = True  # all radio_short_key treads-will-stop
-
-            return
-
-        while not brake_loop:
-            i = 0
-            while not GBase.dict_exit[radio_short_key]:
-                if not fresh_song == stream_song_name:
-
-                    if i == 5:
-                        print(f' \t\t\t\t\t\t(  (  ( (Eisen Radio) )  )  ) )')
-                        i = 0
-                    i += 1
-
-                    fresh_song = stream_song_name  # check if it can be removed, we look in a dict now
-                    clean_name = GBase.remove_special_chars(stream_song_name)
-                    if clean_name == GRecorder.unknown_title_name:  # init name of title on startup
-                        clean_name = clean_name + radio_short_key + '_date_' + GBase.this_time()
-                        clean_name = GBase.remove_special_chars(clean_name)
+            if not display_info == stream_info:
+                if not stream_info == "":
+                    display_info = stream_info
+                    stripped_info = GBase.remove_special_chars(stream_info)
 
                     if first_record:
-                        fresh_file_path = directory_save + '//' + '_incomplete_' + str(clean_name) + str(stream_suffix)
-                        brake_loop = True
-                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path
-                        GRecorder.start_write_command[radio_short_key] = True
-
+                        fresh_file_path = directory_save + '//' + '_incomplete_' + stripped_info + stream_suffix
+                        GRecorder.path_record_dict[str_radio] = fresh_file_path
+                        GRecorder.start_write_command[str_radio] = True
                         first_record = False
                     else:
-                        fresh_file_path = directory_save + '//' + str(clean_name) + str(stream_suffix)
-                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # NEW PATH ... ... ...
+                        fresh_file_path = directory_save + '//' + stripped_info + stream_suffix
+                        GRecorder.path_record_dict[str_radio] = fresh_file_path  # NEW PATH ... ... ...
 
-                    if GBase.dict_exit[radio_short_key]:
-                        # print(f' exit head__ {radio_short_key}')
-                        break
-
-                    while not GBase.dict_exit[radio_short_key]:
-
-                        stream_song_name = GRecorder.current_song_dict[radio_short_key]
-                        if not fresh_song == stream_song_name:
-                            break
-
-                        for sec in range(5):
-                            if GBase.dict_exit[radio_short_key]:
-                                fresh_file_path = directory_save + '//' + '_incomplete_' + clean_name + stream_suffix
-                                GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path
-                                # print(f' exit head_ {radio_short_key}')
-                                break
-                            sleep(1)
+            if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:
+                stripped_info = GBase.remove_special_chars(stream_info)
+                fresh_file_path = directory_save + '//' + '_incomplete_' + stripped_info + stream_suffix
+                GRecorder.path_record_dict[str_radio] = fresh_file_path
+                break
+            sleep(1)
 
     @staticmethod
-    def g_recorder_await_head(key):
-        while not GBase.dict_exit[key]:
+    def g_recorder_make_dir(path_to_save, str_radio):
+        try:
+            GBase.make_directory(path_to_save)
+            return True
+        except FileNotFoundError:
+            GRecorder.current_song_dict[str_radio] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
+            GBase.dict_exit[str_radio] = True  # all str_radio treads-will-stop
+            print('\t---> Recorder Write Failure in : ' + path_to_save)
+            return False
+
+    @staticmethod
+    def g_recorder_await_head(str_radio):
+        while not GBase.dict_exit[str_radio]:
             try:
-                if GRecorder.start_write_command[key]:
+                if GRecorder.start_write_command[str_radio]:
                     break
             except KeyError:
                 pass
@@ -364,19 +340,18 @@ class GRecorder:
                 sleep(.1)
 
     @staticmethod
-    def g_recorder_check_dir(key):
-        # if dir could not be made, no value
+    def g_recorder_path_transfer_test(str_radio):
         try:
-            file_write_path = GRecorder.path_to_song_dict[key]
-            return file_write_path
+            file_path = GRecorder.path_record_dict[str_radio]
+            return file_path
         except KeyError:
-            GRecorder.current_song_dict[key] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
-            GBase.dict_exit[key] = True  # all key treads-will-stop
+            GRecorder.current_song_dict[str_radio] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
+            GBase.dict_exit[str_radio] = True  # all str_radio threads stop, major problem
             return False
 
     @staticmethod
     def g_recorder_request(url):
-        # url was tested before in Gnet
+        # url was tested in Gnet
         try:
             request = GNet.http_pool.request('GET', url,
                                              headers={'Connection': 'keep-alive'},
@@ -390,7 +365,7 @@ class GRecorder:
     @staticmethod
     def g_recorder_next_file(str_radio, full_file_path, record_file, ghetto_recorder, stream_request_size):
         # COMPLETE file path has changed, new file to write; got new metadata
-        if not full_file_path == GRecorder.path_to_song_dict[str_radio]:
+        if not full_file_path == GRecorder.path_record_dict[str_radio]:
             try:
                 if os.path.exists(full_file_path):
                     os.remove(full_file_path)
@@ -405,96 +380,140 @@ class GRecorder:
             record_file.seek(0)
             record_file.truncate()
 
-        full_file_path = GRecorder.path_to_song_dict[str_radio]  # SET NEW PATH (title) after copy
+        full_file_path = GRecorder.path_record_dict[str_radio]  # SET NEW PATH (title) after copy
         return full_file_path
 
     @staticmethod
     def g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_request_size):
         try:
-            record_file.flush()
-            ghetto_size = os.path.getsize(ghetto_recorder)
+            sleep(5)
+            file_size = os.path.getsize(ghetto_recorder)
+            if int(file_size) >= int(stream_request_size):
+                awaited_head_incomplete_tag = GRecorder.path_record_dict[str_radio]
+                shutil.copyfile(ghetto_recorder, awaited_head_incomplete_tag)
 
-            if int(ghetto_size) >= int(stream_request_size):
-                ghetto_copy = GRecorder.path_to_song_dict[str_radio]  # full path last file name
-                print(f'\t "{str_radio}" last file (marked: _incomplete_): '
-                      f'{GRecorder.current_song_dict[str_radio]} ')
-                shutil.copyfile(ghetto_recorder, ghetto_copy)
+            record_file.seek(0)
+            record_file.truncate()
+            record_file.flush()
             record_file.close()
 
             if os.path.exists(ghetto_recorder):
                 os.remove(ghetto_recorder)
-        except UnicodeEncodeError:
-            print(f' {str_radio} copy last file (marked: _incomplete_):')
         except OSError as ex:
             print(ex)
 
+    @staticmethod
+    def g_recorder_empty_queue(str_radio):
+        while not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].empty():
+            GRecorder.audio_stream_queue_dict[str_radio + ',audio'].get()
+
     @staticmethod
     def g_recorder_write_queue(str_radio, chunk):
         if not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].full():
             GRecorder.audio_stream_queue_dict[str_radio + ',audio'].put(chunk)
 
+    @staticmethod
+    def g_recorder_ask_bit_rate(url):
+        try:
+            with urllib.request.urlopen(url, timeout=15) as response:
+                bit_rate = response.getheader('icy-br')
+                return bit_rate
+        except Exception as ex:
+            print(ex)
+            return False
+
+    @staticmethod
+    def g_recorder_calc_chunk_size(url):
+        """avoid digital noise, delays and connection breaks"""
+        stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 4  # 8KB * x; HQ audio 320kB/s
+        if GRecorder.g_recorder_ask_bit_rate(url):
+            bit_rate = int(GRecorder.g_recorder_ask_bit_rate(url))
+            if bit_rate <= 80:
+                stream_chunk_size = io.DEFAULT_BUFFER_SIZE
+            if (bit_rate > 80) and (bit_rate <= 160):
+                stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 2
+            if (bit_rate > 160) and (bit_rate <= 240):
+                stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 3
+        return stream_chunk_size
+
+    @staticmethod
+    def g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path):
+
+        with open(ghetto_recorder, 'wb') as record_file:
+            for chunk in request.stream(stream_chunk_size):
+                record_file.write(chunk)
+                new_file_path = GRecorder.g_recorder_next_file(str_radio,
+                                                               full_file_path,
+                                                               record_file,
+                                                               ghetto_recorder,
+                                                               stream_chunk_size)
+                full_file_path = new_file_path
+
+                if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:  # timer dict_exit
+                    GRecorder.g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_chunk_size)
+                    request.release_conn()
+                    break
+
     @staticmethod
     def ghetto_recorder_tail(url, str_radio, path_to_save, suffix, str_action):
 
-        stream_request_size = io.DEFAULT_BUFFER_SIZE * 4
-        audio_stream_queue = queue.Queue(maxsize=20)  # 8KB * 4 * 20
+        audio_stream_queue = queue.Queue(maxsize=10)  # maxsize for safety, normal one in use
+        stream_chunk_size = GRecorder.g_recorder_calc_chunk_size(url)
+
         GRecorder.audio_stream_queue_dict[str_radio + ',audio'] = audio_stream_queue
-        ghetto_recorder = path_to_save + '//__ghetto_recorder' + str(suffix)
+        ghetto_recorder = os.path.join(path_to_save, '__ghetto_recorder' + str(suffix))
+        request = GRecorder.g_recorder_request(url)
 
-        GRecorder.g_recorder_await_head(str_radio)
+        if str_action == "record":
+            GRecorder.g_recorder_await_head(str_radio)
+            full_file_path = GRecorder.g_recorder_path_transfer_test(str_radio)
+            if not full_file_path:
+                return
+            if not GRecorder.g_recorder_make_dir(path_to_save, str_radio):
+                return
+            GRecorder.g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path)
 
-        # for thread in threading.enumerate():
-        #     print(thread.name)
+        # k = 0
+        if str_action == "listen":
+            for chunk in request.stream(stream_chunk_size):
+                # if k % 10 == 0:
+                #     for thread in threading.enumerate():
+                #         print(f' {thread.name}   {k}')
+                # k += 1
 
-        full_file_path = GRecorder.g_recorder_check_dir(str_radio)
-        if full_file_path:
-            request = GRecorder.g_recorder_request(url)
-
-            if str_action == "record":
-                with open(ghetto_recorder, 'wb') as record_file:
-                    for chunk in request.stream(stream_request_size):
-                        record_file.write(chunk)
-                        full_file_path = GRecorder.g_recorder_next_file(str_radio,
-                                                                        full_file_path,
-                                                                        record_file,
-                                                                        ghetto_recorder,
-                                                                        stream_request_size)
-
-                        if not GRecorder.record_active_dict[str_radio]:
-                            GRecorder.g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_request_size)
-                            break
-
-            if str_action == "listen":
-                for chunk in request.stream(stream_request_size):
-                    GRecorder.g_recorder_write_queue(str_radio, chunk)
-                    if not GRecorder.listen_active_dict[str_radio]:
-                        break
+                GRecorder.g_recorder_write_queue(str_radio, chunk)
+                if not GRecorder.listen_active_dict[str_radio] or GBase.dict_exit[str_radio]:
+                    GRecorder.g_recorder_empty_queue(str_radio)
+                    request.release_conn()
+                    break
 
     @staticmethod
     def metadata_header_info(request, str_radio, request_time):
         try:
             GRecorder.ghetto_measure[str_radio + ',request_time'] = request_time
             GRecorder.ghetto_measure[str_radio + ',suffix'] = request.headers['content-type']
+        except KeyError:
+            pass
+        try:
             GRecorder.ghetto_measure[str_radio + ",icy_genre"] = request.headers["icy-genre"]
             GRecorder.ghetto_measure[str_radio + ",icy_name"] = request.headers["icy-name"]
             GRecorder.ghetto_measure[str_radio + ",icy_br"] = request.headers["icy-br"]
+            GRecorder.ghetto_measure[str_radio + ",icy_url"] = request.headers["icy-url"]
         except KeyError:
             pass
 
     @staticmethod
     def metadata_icy_info(request):
         try:
-            icy_metadata = request.headers['icy-metaint']
-            icy_metadata = int(icy_metadata)
-
-            request.read(icy_metadata)
-            chunk_1b = request.read(1)
-            chunk_1b = ord(chunk_1b)
-            read_bytes = chunk_1b * 16
-            read_bytes = int(read_bytes)
-            metadata_content = request.read(read_bytes)
+            icy_meta_int = int(request.headers['icy-metaint'])
+            request.read(icy_meta_int)
+            start_byte = request.read(1)
+            start_int = ord(start_byte)
+            num_of_bytes = start_int * 16
+            metadata_content = request.read(num_of_bytes)
             return metadata_content
-        except KeyError:
+        except Exception as e:
+            print(e)
             return False
 
     @staticmethod
@@ -509,41 +528,41 @@ class GRecorder:
     def metadata_get_display_extract(icy_info):
         # StreamTitle='X-Dream - Panic In Paradise * anima.sknt.ru';StreamUrl='';
         try:
-            title_info = icy_info.split(";")
-            if not len(title_info) > 1 or title_info is None:
-                return    # empty list
-            title_info = title_info[0].split("=")
-            title = str(title_info[1])
+            title_list = icy_info.split(";")
+            if not len(title_list) > 1 or title_list is None:
+                return  # empty list
+            title_list = title_list[0].split("=")
+            title = str(title_list[1])
             title = GBase.remove_special_chars(title)
-            return title
+            if title is not None:
+                return title
         except IndexError:
             pass
         except OSError:
             pass
+        return
 
     @staticmethod
     def metadata_get_display_info(icy_info):
-        # <class 'bytes'> decode to <class 'str'>
-        title = ''
-        try:
-            title = GRecorder.metadata_get_display_extract(icy_info.decode('utf-8'))
-        except AttributeError:
-            pass
+        # <class 'bytes'> decode to <class 'str'> actually b''
+        title = GRecorder.metadata_get_display_extract(icy_info.decode('utf-8'))
         if not title:
             return
         if title:
             return title
 
     @staticmethod
-    def get_metadata_from_stream_loop(url, str_radio):
+    def get_metadata_from_stream_loop(url, str_radio, str_action):
+        stop_while = False
         icy_info = ''
-        while not GBase.dict_exit[str_radio]:
+        while 1:
             try:
                 start_time = time()
                 response = GRecorder.metadata_request(url)
                 request_time = round((time() - start_time) * 1000)
                 GRecorder.metadata_header_info(response, str_radio, request_time)
                 icy_info = GRecorder.metadata_icy_info(response)
+                response.release_conn()  # preload_content=False
             except HTTPError:
                 pass
             except URLError:
@@ -554,7 +573,7 @@ class GRecorder:
             if title:
                 try:
                     if title[0] == "'" and title[-1] == "'":
-                        GRecorder.current_song_dict[str_radio] = title[1:-1]   # sequence drop 0 and last char
+                        GRecorder.current_song_dict[str_radio] = title[1:-1]  # |sequence, ex 0 and last char |0 |1 |²2
                     else:
                         GRecorder.current_song_dict[str_radio] = title
                 except KeyError:
@@ -564,9 +583,17 @@ class GRecorder:
                     pass
 
             for sec in range(2):
-                if GBase.dict_exit[str_radio]:
-                    break     # while
+                if str_action == "listen":
+                    if GBase.dict_exit[str_radio] or not GRecorder.listen_active_dict[str_radio]:
+                        stop_while = True
+                        break
+                if str_action == "record":
+                    if GBase.dict_exit[str_radio] or not GRecorder.record_active_dict[str_radio]:
+                        stop_while = True
+                        break
                 sleep(1)
+            if stop_while:
+                break
 
     @staticmethod
     def playlist_m3u(url, str_radio):
@@ -617,27 +644,33 @@ def check_alive_playlist_container(str_radio, str_url):
         return False
 
 
-def record(str_radio, url, str_action):
-    stream_suffix = GNet.stream_filetype_url(url, str_radio)
-    GRecorder.current_song_dict[str_radio] = GRecorder.unknown_title_name
-    GIni.start_stop_recording[str_radio] = 'start'  # remnant of hit keyword, start record in tk/tcl g.recorder
-    # GIni.start_stop_recording[str_radio] = 'stop'   # init it here, checkbox user interface
-    GIni.start_stop_recording[str_radio + '_adv'] = 'start_from_here'
-
-    dir_save = GBase.radio_base_dir + '//' + str_radio
-
+def record_start_radio(str_radio, url, stream_suffix, dir_save, str_action):
     threading.Thread(name=str_radio + '_' + str_action + "_show", target=GRecorder.ghetto_recorder_display_title,
-                     args=(url, str_radio),
+                     args=(str_radio, str_action),
                      daemon=True).start()
     threading.Thread(name=str_radio + '_' + str_action + "_head", target=GRecorder.ghetto_recorder_head,
-                     args=(dir_save, stream_suffix, str_radio),
+                     args=(dir_save, stream_suffix, str_radio, str_action),
                      daemon=True).start()
     threading.Thread(name=str_radio + '_' + str_action + "_tail", target=GRecorder.ghetto_recorder_tail,
                      args=(url, str_radio, dir_save, stream_suffix, str_action),
                      daemon=True).start()
     threading.Thread(name=str_radio + '_' + str_action + "_meta", target=GRecorder.get_metadata_from_stream_loop,
-                     args=(url, str_radio),
+                     args=(url, str_radio, str_action),
                      daemon=True).start()
 
+
+def record(str_radio, url, str_action):
+    stream_suffix = GNet.stream_filetype_url(url, str_radio)
+    if str_action == "record":
+        GRecorder.current_song_dict[str_radio] = GRecorder.unknown_title_name
+    if str_action == "listen":
+        GRecorder.current_song_dict[str_radio] = ""
+    GIni.start_stop_recording[str_radio] = 'start'  # remnant of hit keyword, start record in tk/tcl g.recorder
+    # GIni.start_stop_recording[str_radio] = 'stop'   # init it here, checkbox user interface
+    GIni.start_stop_recording[str_radio + '_adv'] = 'start_from_here'
+
+    dir_save = GBase.radio_base_dir + '//' + str_radio  # [Baustelle]
+    record_start_radio(str_radio, url, stream_suffix, dir_save, str_action)
+
     # ################################## end ########################################
     # this version ends here. no loop
